package jprobe;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

  /**
   * Main entry point for JProbe.
   * JProbe is a wrapper for MainFrame, which does most of the work.
   */
   public class JProbe 
{
  /**
   * Set this to print thread info
   */
  public static final boolean printThread=false;
  

  /**
   * Constructor for JProbe.  Creates the MainFrame.
   */
  public JProbe()
  {
    if(printThread) {
      System.out.println("JProbe: " +
       Thread.currentThread().toString());
    }

    JFrame frame = new MainFrame();
    
    // Set window decorations
    frame.setDefaultLookAndFeelDecorated(true);
    
    // Make it exit when the window manager close button is clicked
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    
    // Center it on the screen
    frame.setLocationRelativeTo(null);
    
    // Display the MainFrame
    frame.pack();
    frame.setVisible(true);
  }

  /**
   * Main program for JProbe
   * @param args arguments
   */
  public static void main(String[] args)
  {

    if(JProbe.printThread) {
      System.out.println("main: " +
       Thread.currentThread().toString());
    }

    try
    {
      // Set the native loook and feel
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      
      // Make the job run in the AWT thread
      SwingUtilities.invokeLater(new Runnable () {
        public void run() {
          // Start a JProbe, which will start a MainFrame
          new JProbe();
       }});
    }
    catch(Throwable t)
    {
      t.printStackTrace();
    }

    if(JProbe.printThread) {
      System.out.println("main [end]: " +
       Thread.currentThread().toString());
    }
  }

}