package jprobe;

//import javax.sql.ConnectionEvent;
import gov.aps.jca.*;
import gov.aps.jca.dbr.*;
import gov.aps.jca.event.ConnectionEvent;
import gov.aps.jca.event.ConnectionListener;
import gov.aps.jca.event.MonitorEvent;
import gov.aps.jca.event.MonitorListener;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.BevelBorder;

/**
 * Main frame for JProbe
 */
public class MainFrame extends JFrame 
{
  // Improve readability
  private static final boolean OK=true;
  private static final boolean ERR=false;
  
  private ProbeConnectionListener connectionListener=null;
  private ProbeMonitorListener monitorListener=null;
  private JCALibrary jca=null;
  private Context ctxt=null;
  private Channel chan=null;
  private String name=null;
  private String [] value=null;
  private int connectionCounter=0;
  private int monitorCounter=0;

  private JLabel statusBar = new JLabel();
  private JMenuItem menuHelpAbout = new JMenuItem();
  private JMenu menuHelp = new JMenu();
  private JMenuItem menuFileExit = new JMenuItem();
  private JMenu menuFile = new JMenu();
  private JMenuBar menuBar = new JMenuBar();
  private JPanel panelCenter = new JPanel();
  private JLabel Value = new JLabel();
  private JLabel jLabel2 = new JLabel();
  private JTextField Name = new JTextField();
  private JLabel jLabel1 = new JLabel();
  private BorderLayout layoutMain1 = new BorderLayout();
  private JButton infoButton = new JButton();
  private GridBagLayout gridBagLayout1 = new GridBagLayout();
  private JButton adjustButton = new JButton();

  /**
   * Default constructor
   */
  public MainFrame()
  {
    try {
      jbInit();
      init();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Swing initialization
   * @throws java.lang.Exception
   */
  private void jbInit() throws Exception
  {
    if(JProbe.printThread) {
      System.out.println("jbInit: " + Thread.currentThread().toString());
    }
    this.setJMenuBar(menuBar);
    this.getContentPane().setLayout(gridBagLayout1);
    panelCenter.setLayout(null);
    Value.setText("");
    Value.setBounds(new Rectangle(75, 40, 235, 20));
    jLabel2.setText("Value:");
    jLabel2.setBounds(new Rectangle(10, 40, 85, 20));
    Name.setBounds(new Rectangle(75, 15, 235, 20));
    Name.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          NameOnActionPerformed(e);
        }
      });
    jLabel1.setText("PV Name:");
    jLabel1.setBounds(new Rectangle(10, 15, 85, 20));
    infoButton.setText("Info");
    infoButton.setBounds(new Rectangle(190, 65, 70, 25));
    infoButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          infoButtonOnActionPerformed(e);
        }
      });
    adjustButton.setText("Adjust");
    adjustButton.setBounds(new Rectangle(65, 65, 70, 25));
    adjustButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          adjustButtonOnActionPerformed(e);
        }
      });
    this.setSize(new Dimension(334, 168));
    this.setTitle("JProbe");
    this.setBounds(new Rectangle(10, 10, 334, 180));
    menuFile.setText("File");
    menuFileExit.setText("Exit");
    menuFileExit.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent ae)
        {
          fileExitOnActionPerformed(ae);
        }
      });
    menuHelp.setText("Help");
    menuHelpAbout.setText("About");
    menuHelpAbout.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent ae)
        {
          helpAboutOnActionPerformed(ae);
        }
      });
    statusBar.setText("Waiting for PV name");
    statusBar.setToolTipText("Status");
    statusBar.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
    menuFile.add(menuFileExit);
    menuBar.add(menuFile);
    menuHelp.add(menuHelpAbout);
    menuBar.add(menuHelp);
    this.getContentPane().add(statusBar, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 223, 0));
    panelCenter.add(adjustButton, null);
    panelCenter.add(infoButton, null);
    panelCenter.add(jLabel1, null);
    panelCenter.add(Name, null);
    panelCenter.add(jLabel2, null);
    panelCenter.add(Value, null);
    this.getContentPane().add(panelCenter, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 323, 127));
  }
  
  /**
   * Implementation of Connection Listener class
   */
  private class ProbeConnectionListener implements ConnectionListener {
    public void connectionChanged(ConnectionEvent ev) {
      onConnectionChanged(ev);
    }
  };

  /**
   * Implementation of MonitorListener class
   */
  private class ProbeMonitorListener implements MonitorListener {
    /**
     * 
     */
    public void monitorChanged(MonitorEvent ev) {
      onValueChanged(ev);
    }
  };

  // Functions ////////////////////////////////////////////////////////////////

  /**
   * JCA Initialization
   * @return success or failure
   */
  private boolean init() {
    boolean retVal=OK;
    
  // Initialize JCA
    updateStatus("Initializing");
    try {
      // Get the JCALibrary instance
      jca=JCALibrary.getInstance();

      // Create a thread safe context with default configuration values
      // This is for JCA
      // ctxt=jca.createContext(JCALibrary.JNI_THREAD_SAFE);
      // This is for CAJ
      ctxt=jca.createContext("com.cosylab.epics.caj.CAJContext");

      updateStatus("Initialization complete");
    } catch(Exception ex) {
      errMsg("Error initializing: " + ex.getMessage());
      updateStatus("Error initializing");
      retVal=ERR;
    }
    
    return retVal;
  }
  
  /**
   * Opens a new channel, closing any old one first
   * @param newName
   * @return success or failure
   */
  private boolean newChannel(String newName) {
    boolean retVal=OK;
    
    if(JProbe.printThread) {
      System.out.println("newChannel: " + Thread.currentThread().toString());
    }

    // Check the name
    if(newName == null || newName.equals("")) {
      errMsg("No name specified");
      return ERR;
    }
    
    // Close a previous channel
    if(!isClosed()) closeChannel();
    name=newName;
    
    // Create a new channel
    try {
      updateStatus("Searching");
      connectionListener=new ProbeConnectionListener();
      chan=ctxt.createChannel(name,connectionListener);
      ctxt.flushIO();
    } catch(Exception ex) {
      errMsg("Error creating channel: " + ex.getMessage());
      retVal=ERR;
    }
    return retVal;
  }
  
  /**
   * Closes a channel and releases resources
   * @return success or failure
   */
  private boolean closeChannel() {
    boolean retVal=OK;
    
    if(JProbe.printThread) {
      System.out.println("closeChannel: " + Thread.currentThread().toString());
    }
    
    try {
      // Clear the channel (Assume this removes the listeners)
      if(chan != null) chan.destroy();
    } catch(Exception ex) {
      errMsg("Error closing channel: " + ex.getMessage());
      retVal=ERR;
    } finally {
      connectionListener=null;
      monitorListener=null;
      chan=null;
      name=null;
      value=null;
      connectionCounter=0;
      monitorCounter=0;
    }
    return retVal;
  }
  
  /**
   * Updates the status bar
   * @param s Status bar text
   */
  private void updateStatus(final String s) {
    if(JProbe.printThread) {
      System.out.println("updateStatus: " +
       Thread.currentThread().toString() + ": " + s);
    }

    if(s != null) {
      // Set this to execute setText in the AWT thread
      final boolean later=true;
      if(later) {
        // Make it run in the AWT thread
        SwingUtilities.invokeLater(new Runnable () {
          public void run() {
            if(JProbe.printThread) {
              System.out.println("statusBar.setText: " +
               Thread.currentThread().toString() + ": " + s);
            }
            statusBar.setText(s);
         }});
      } else {
        // Run it in this thread
        statusBar.setText(s);
      }
    }
  }
  
  /**
   * Quits the application
   */
  private void quit() {
    int retVal=0;

    if(JProbe.printThread) {
      System.out.println("quit: " + Thread.currentThread().toString());
    }
    
    if(jca != null && ctxt != null) {
      try {
        // Clear the channel
        closeChannel();

        // Destroy the context
        ctxt.destroy();
        
      } catch(Exception ex) {
        errMsg("Clean up failed: " + ex.getMessage());
        retVal=1;
      }
    }
    System.exit(retVal);
  }
  
  /**
   * Error message dialog
   * @param err
   */
  public void errMsg(String err) {
    // Show it in a message box
    JOptionPane.showMessageDialog(this,err,
    "Error",JOptionPane.ERROR_MESSAGE);
  }

  /**
   * Connection callback
   * @param ev
   */
  private void onConnectionChanged(ConnectionEvent ev) {
    Channel ch= (Channel)ev.getSource();
    Context ctxt= ch.getContext();
    
    if(JProbe.printThread) {
      System.out.println("onConnectionChanged: " +
       Thread.currentThread().toString());
    }

    connectionCounter++;

    // Start a monitor on the first connection    
    if(connectionCounter == 1 && ch.getConnectionState() == Channel.CONNECTED) {
      // This is the first connection.
      try {
        updateStatus("Search successful for: " + getName());

      // Add a monitor listener
        monitorListener=new ProbeMonitorListener();
        ch.addMonitor(DBRType.STRING,1,Monitor.VALUE|Monitor.LOG|Monitor.ALARM,
         monitorListener);
        ctxt.flushIO();
      } catch(Exception ex) {
      connectionCounter=0;
        ex.printStackTrace();
      }
    }

    // Branch on connection state
    if(ch.getConnectionState() == Channel.CONNECTED) {
      updateStatus("Connected and Montoring");
      connectionCounter++;
    } else if(ch.getConnectionState() == Channel.CLOSED) {
      updateStatus("Closed");
    } else if(ch.getConnectionState() == Channel.DISCONNECTED) {
      updateStatus("Disconnected");
    } else if(ch.getConnectionState() == Channel.NEVER_CONNECTED) {
      updateStatus("Never connected");
    } else {
      // Shouldn't happen
      updateStatus("Unknown state");
    }
  }

   /**
   * Monitor callback
   * @param ev
   */
  private void onValueChanged(MonitorEvent ev) {
    Channel ch= (Channel) ev.getSource();
    Context ctxt= ch.getContext();

    if(JProbe.printThread) {
      System.out.println("onValueChanged: " +
       Thread.currentThread().toString());
    }
 
    monitorCounter++;

  // Check the status
    if (ev.getStatus() != CAStatus.NORMAL) {
      System.err.println("monitorChanged: Bad status for: " +
       getName());
    }
    
    // Get the value from the DBR
    try {
      DBR dbr= ev.getDBR();
      value=((STRING)dbr).getStringValue();
      Value.setText(value[0]);
    } catch(Exception ex) {
      System.err.println("monitorChanged: Bad value for " +
      ch.getName() + ":\n " + ex);
      return;
    }
  }
  
  // Accessors ////////////////////////////////////////////////////////////////
  
  /**
   * Get process variable name
   * @return process variable name
   */
  public String getName() {
    return name;
  }
  
  /**
   * Get the value
   * @return value
   */
  public String getValue() {
    return value[0];
  }

  /**
   * Get whether the channel is closed or not
   * @return true if closed
   */
  public boolean isClosed() {
    return (connectionListener == null  && 
      monitorListener == null &&
      chan == null &&
      name == null &&
      value == null &&
      connectionCounter == 0 &&
      monitorCounter == 0);
  }
  
  // Action Handlers //////////////////////////////////////////////////////////

  /**
   * File | Exit handler
   * @param e
   */
  private void fileExitOnActionPerformed(ActionEvent e)
  {
    if(JProbe.printThread) {
      System.out.println("fileExitOnActionPerformed: " +
       Thread.currentThread().toString());
    }
 
    quit();
  }

  /**
   * Help | About handler
   * @param e
   */
  private void helpAboutOnActionPerformed(ActionEvent e)
  {

    if(JProbe.printThread) {
      System.out.println("helpAboutOnActionPerformed: " +
       Thread.currentThread().toString());
    }

    JOptionPane.showMessageDialog(this, new AboutBoxPanel(), "About",
     JOptionPane.PLAIN_MESSAGE);
  }

  /**
   * Name entry handler
   * @param e
   */
  private void NameOnActionPerformed(ActionEvent e)
  {
    String newName=Name.getText();
    Value.setText("");
    
    if(JProbe.printThread) {
      System.out.println("NameOnActionPerformed: " +
       Thread.currentThread().toString());
    }
   
    boolean status=newChannel(newName);
    if(status == OK) {
      updateStatus("Waiting for connection");
    } else {
      updateStatus("Error starting " + newName);
    }
  }

  /**
   * Info button handler
   * @param e
   */
  private void infoButtonOnActionPerformed(ActionEvent e) {
  
    if(JProbe.printThread) {
      System.out.println("infoButtonOnActionPerformed: " +
       Thread.currentThread().toString());
    }
  
    try {
      String info="No information available";
      if(chan != null) {
        if(chan.getConnectionState() == Channel.CONNECTED) {
          info="CHANNEL: " + chan.getName() + "\n";
          info+="TYPE: " + chan.getFieldType() + "\n";
          info+="COUNT: " + chan.getElementCount() + "\n";
          info+="STATE: " + chan.getConnectionState() + "\n";
          info+="HOSTNAME: " + chan.getHostName() + "\n";
          info+="READ: " + chan.getReadAccess() + "\n";
          info+="WRITE: " + chan.getWriteAccess();
        } else {
          info="Not connected";
        }
      }
      String title="Information for " + ((getName() != null)?getName():"");
      JOptionPane.showMessageDialog(this,info,title,
       JOptionPane.INFORMATION_MESSAGE);
    } catch(Exception ex) {
        errMsg("Get info failed: " + ex.getMessage());
    }
  }

  /**
   * Adjust button handler
   * @param e
   */
  private void adjustButtonOnActionPerformed(ActionEvent e) {

    if(JProbe.printThread) {
      System.out.println("adjustButtonOnActionPerformed: " +
       Thread.currentThread().toString());
    }

    try {
      if(chan != null) {
        if(chan.getConnectionState() == Channel.CONNECTED) {
          String s = (String)JOptionPane.showInputDialog(this,
           "Enter new value:","New Value",
           JOptionPane.PLAIN_MESSAGE,
           null,null, getValue());
          if ((s != null) && (s.length() > 0)) {
            chan.put(s);
            ctxt.flushIO();
          }
        } else {
          errMsg("Not connected");
        }
      } else {
        errMsg("No channel");
      }
    } catch(Exception ex) {
        errMsg("Adjust failed: " + ex.getMessage());
    }
  }

}
